PROGRAM JAC_TEST
!
! TEST JACOBI ITERATION ROUTINE (only for SYMMETRIC Matrix)
!
      IMPLICIT NONE
!
!  Set matirx dimensions
!
      INTEGER NDIM       ! Maximum matrix dimension
      PARAMETER(NDIM=12)
!
!  Declare variables
!
      REAL*8 A(NDIM,NDIM)  ! Input matrix to be transformed
      REAL*8 B(NDIM,NDIM)  ! Original input matrix
      REAL*8 R(NDIM,NDIM)  ! Rotation matrix
      REAL*8 EPS           ! Maximum error measure for convergence
      INTEGER N            ! Size of matrix
      INTEGER MAXE         ! Maximum number of iterations
      INTEGER I,J          ! Counters
!
!  Set the input matrix and parameters
!
      N=3
      OPEN(UNIT=10,FILE="ELC-matrix",STATUS='OLD',ACTION='READ',err=3000)
      DO I=1,N
            READ(10,*) (A(I,J),J=1,N)
      ENDDO

      EPS=1.E-8
      MAXE=100
!
!  Save the original input matrix in B
!
      DO I=1,N
         DO J=1,N
            B(I,J)=A(I,J)
         END DO
      END DO
!
!  Perform Jacobi iteration for eigenvalues and eigenvectors
!
!      WRITE(*,*) MAXE
      CALL Jacobi(A,R,N,NDIM,EPS,MAXE)
!
!  Write input marix
!     
!      WRITE (*,*) 'Input elastic constants matrix' 
!      DO I=1,N
!         DO J=1,N
!            WRITE(*,'(A,I1,A,I1,A,1PE10.3)') 'B(',I,',',J,')=',B(I,J)
!         END DO
!      END DO
!
!  Write the rotation (row-wise eigenvectors) matrix
!
!      WRITE (*,*) 'Row-Wise eigenvectors matrix' 

!      DO I=1,N
!         DO J=1,N
!            WRITE(*,'(A,I1,A,I1,A,1PE10.3)') 'R(',I,',',J,')=',R(I,J)
!         END DO
!      END DO
!
!  Write eigenvalue matrix with residuals
!
!      WRITE (*,*) 'Eigenvalue matrix' 
!      DO I=1,N
!         DO J=1,N
!            WRITE(*,'(A,I1,A,I1,A,1PE10.3)') 'A(',I,',',J,')=',A(I,J)
!          END DO
!      END DO
!-------------------------------------------------------------
!-------------------------------------------------------------
!                Elastic Stability Conditions             
!    The generic necessary and sufficient criterion that  
!    all eigenvalues of A be position                     
!                        REFERENCE                        
!       Mouhat F, Coudert F X. Necessary and sufficient   
!       elastic stability conditions in various crystal   
!       systems[J]. Physical Review B, 2014, 90(22).      
!-------------------------------------------------------------

!  I here is a countor. when A(J,J) > 0 then I=I+1 
! if I=4 i.e. all of A(J,J) of A-matrix are positive else
! all of A(J,J) of A-matrix are not positive.
! A is 3*3 matrix
         I=1
         DO J=1,N
            if (A(J,J).GT.0.0) then
             I=I+1
            end if
!          write(*,*) 'Eigen Values'
!          write (*,*) A(J,J)
          END DO
        if (I.EQ.4) THEN
                write(*,*) '-------------------------------------------'
                write(*,*) 'Elastic Stability Conditions are satisfied'
                write(*,*) '-------------------------------------------'

        else 
                write(*,*) '-----------------------------------------------'
                write(*,*) 'Elastic Stability Conditions are not satisfied'
                write(*,*) '-----------------------------------------------'

        end if


      GOTO 3001

 3000 WRITE(*,*) ' '  
      WRITE(*,*) 'No valid ELC-matrix file to check Elastic Stability Conditions.'
      WRITE(*,*) ' '  
      STOP
 3001 END

!---------------------------------------------------------------------------
SUBROUTINE Jacobi(A,R,N,NN,EPS,MAXE)   
!                                                                        
! Find eigenvectors & eigenvalues by Jacobi iteration with 'max' search      
! A is input matrix for eigenvalue analysis and must be symmetric        
! A is NxN                                                               
! Eps is off diagonal norm toleren!e, MAXE is the maximum no. of iteration
! R is the matrix of eigenvectors stored rowwise                         
! At the end A is output with eigenvalues on diagonal and residuals off diagonal      
!                      
      IMPLICIT NONE
!
!  Declare variables
!
      INTEGER N,NN,MAXE,Knt,I,J
      REAL*8 EPS,E,D,Apq,E0,D0
      REAL*8 A(NN,NN),R(NN,NN)
!                                                                        !
! Begin Iteration counter
!
      Knt=1
!
!  Find original norms
!         
      WRITE(*,*) 'JACOBI'                                                           
      CALL Jnorm(A,N,NN,E0,D0)                                     
!
!  Set the origianl rotation matrix to the identity matrix
!
      DO I=1,N                                                             
         DO J=1,N                                                             
            R(I,J)=0.                                                         
         END DO                                                            
         R(I,I)=1.                                                        
      END DO
!
!  A 1x1 is automatically the eigenvalue
!                                                                    
      IF (N.EQ.1) RETURN          
!
! If A is diagonal it is the eigenvalue matrix
!
      IF (E0.LE.0.) RETURN
!
!  Write the column headings for the convergence summary
!
!      WRITE(*,'(1X,A,6X,A)') 'Iteration','Error'
!
!  Loop over Jacobi iterations
!
      DO Knt=1,MAXE 
!     WRITE(*,*) 'JACOBI2'                                                           
        
         CALL Iter(A,R,N,NN,Apq)  ! Perform an iteration
         CALL Jnorm(A,N,NN,E0,D0)   ! Find the new norms
!
!  Write the convergen!e summary
!                                
!         WRITE(*,'(I10,1PE15.3)') Knt,E0
         IF (E0/D0.LT.EPS) THEN ! Convergence Has Occurred
!            WRITE(*,'(1X,A,6X,A)') 'Last Iteration','Error'
!            WRITE(*,'(I10,1PE15.3)') Knt,E0

            RETURN
         END IF
      END DO
!
!  Exiting the loop means there were too many iteration.  Write the message.
      WRITE(*,*) 'ERROR: Jacobi iteration did not converge for',EPS            
      WRITE(*,*) ' in less than ',MAXE,'iterations'
      write(*,*) ' '
      write(*,*) '-------------------------------------------------'
      write(*,*) 'Please check Elastic Stability Conditions by hand'
      write(*,*) '-------------------------------------------------'
      write(*,*) ' '
!  Too many iterations return
!
      RETURN
      END                                                                   
!-----------------------------------------------------------------------------

SUBROUTINE Iter(A,R,N,NN,Apq)                                          
!
!  Perform a single Jacobi iteration
!
      IMPLICIT NONE
!
!  Declare variables
!
      INTEGER P,Q     ! Indices of maximum off diagonal term
      INTEGER N       ! Size of input matrix
      INTEGER NN      ! Dimension of input marix
      INTEGER I,J     ! Counters
      REAL*8 R1(2,2)  ! 2x2 rotation matrix
      REAL*8 A(NN,NN) ! Input matrix
      REAL*8 Apq      ! Maximum offdiagonal element
      REAL*8 Th       ! Rotation angle
      REAL*8 R(NN,NN) ! NXN Rotation matrix
!
!  Find the maximum off-diagonal term and row(p) and column(q) of it
!                        
      CALL Search(A,N,NN,P,Q)         
      Apq=A(P,Q)                                                               
!
!  Find the corresponding angle
!
      CALL Angle(A,P,Q,Th,N,NN)
!
!  Form the 2X2 matrix(R1) rotation matrix
!                                                 
      CALL Formr(A,R1,P,Q,Th,N,NN)
!
!  Update R to the new NXN rotation matrix
!                                            
      CALL Newr(R,R1,N,NN,P,Q)                                              
!
!  Form the rotated A(new)=R1(transpose)*A(old)*R1
!
      CALL Newa(A,R1,N,NN,P,Q)                                              
!
!  Normal return
!
      RETURN
      END 
                                                              
!-------------------------------------------------------------------
SUBROUTINE Search(A,N,NN,P,Q)
!
!  Find the location of the maximum off-diagonal element
!
      IMPLICIT NONE
!
!  Declare variables
!
      INTEGER P,Q     ! Location of maximum element
      INTEGER N       ! Size of A
      INTEGER NN      ! Dimension of A
      INTEGER I,J     ! Counters
      REAL*8 A(NN,NN) ! Current inpt matrix
      REAL*8 Maxi      ! Current maximum off-diagonal term
!
!  Initialize maximum to 0, and P & Q to first off diagonal element to check
!
      Maxi=0                                                                    
      P=1                                                                      
      Q=2                                                                      
      DO I=1,N-1 
!
!  Loop over all off-diagonal elements to find maximum
!  Find Absolute maximum value.
!  As M. Jamal checked,If we start with Absolute maximum value 
!  the iteration for finding eigenvalues terminates soon.                                                        
         DO J=I+1,N                                                           
            IF (DABS(A(I,J)).GT.Maxi) THEN ! Update maximum                                    
               P=I                                                             
               Q=J                                                              
               Maxi=DABS(A(I,J))     
            END IF                                               
         END DO                                                                
      END DO                        
!
!  Normal maximum
!                                            
      RETURN
      END       
                                                            
!---------------------------------------------------------------------------
SUBROUTINE Angle(A,P,Q,Th,N,NN)                                          
!
!  Find the rotation angle from the maximum off-diagonal term
!
      IMPLICIT NONE
!
!  Declare variables
!
      INTEGER P,Q        ! Location of maximum off-diagonal term
      INTEGER N          ! Size of input matrix
      INTEGER NN         ! Dimension of input matrix
      REAL*8 A(NN,NN)    ! Input matrix
      REAL*8 Th          ! Rotation matrix
      REAL*8 Td,Tn       ! Numerator and denominator of tangent of angle
      REAL*8 FNItan      ! 2D inverse tangent
      Tn=2.*A(P,Q)       ! Find numerator                                                       
      Td=A(P,P)-A(Q,Q)   ! Find denominator                                                       
      Th=.5*FNItan(Tn,Td)! Find angle
!
!  Normal return
!                                                      
      RETURN
      END                                                                   

!------------------------------------------------------------------------------
REAL*8 FUNCTION FNItan(Y,X)
!
!  Find the inverse tangent given the numerator and denominator of the tangent
!                                                
      IMPLICIT NONE
!
!  Declare variables
!
      REAL*8 Y,X  ! Numerator and denominator
      REAL*8 PI    ! Pi (~22/7)
      REAL*8 At   ! Absolute value of tangent
!
!  Set pi
!
      PI=3.14159265357989
!
!  There is no angle defined at the origin
!
      IF (X.EQ.0..AND.Y.EQ.0.) THEN
         WRITE(6,*)
         WRITE(6,*) '**********************************************'    
         WRITE(6,*) '***ERROR:  Inverse tangent of 0/0 in FNitan***'
         WRITE(6,*) '**********************************************'
         WRITE(6,*)
         Fnitan=0.
      ELSE                                                                     
         FNItan=0                 ! Angle on +x axis                                                             
         IF (Y.EQ.0..AND.X.GT.0.) RETURN                                      
         FNItan=PI                !  Angle on -x axis                                             
         IF (Y.EQ.0..AND.X.LT.0.) RETURN                                       
         FNItan=PI/2.             !  Angle on +y axis                                           
         IF (X.EQ.0..AND.Y.GT.0.) RETURN                                      
         FNItan=3.*PI/2.          !  Angle on -y axis                                            
         IF (X.EQ.0..AND.Y.LT.0.) RETURN
!
!  Angle not on an axis
!                                       
         At=Y/X                    ! Absolute value of tangent                                                           
         FNItan=DABS(DATAN(At))                                                  
         IF (X.GT.0..AND.Y.GT.0.)  RETURN      ! Angle in quadrant 1                                      
         IF (X.GT.0..AND.Y.LT.0.) THEN 
               FNItan=-FNItan                  ! Angle in quadrant 2                          
            ELSE IF (X.LT.0..AND.Y.GT.0.) THEN 
               FNItan=PI-FNItan                ! Angle in quadrant 3                          
            ELSE IF (X.LT.0..AND.Y.LT.0.) THEN 
               FNItan=-(PI-FNItan)             ! Angle in quadrant 4                          
         END IF
      END IF
!
!  Normal return
!
      RETURN
      END         

!--------------------------------------------------------------------------
SUBROUTINE Formr(A,R1,P,Q,Th,N,NN)                                       
!
!  Form the new rotation matrix(R1).
!  If you want to write your program as
!  A(new)=R1(transpose)*A(old)*R1 for finding  eigenvalue
!  therefore R1 is an identical matrix with four of its
!  elements modified
!      R1(q,q)=COS(Th)                                                          
!      R1(p,p)=R1(1,1)                                                          
!      R1(q,p)=SIN(Th)                                                          
!      R1(p,q)=-R1(q,p)

      IMPLICIT NONE
!
!  Declare variables
!
      INTEGER P,Q      ! Location of maximum off-diagonal term
      INTEGER N        ! Size of input matrix
      INTEGER NN       ! Dimension of matrix
      REAL*8 A(NN,NN)  ! Input matrix
      REAL*8 R1(2,2)   ! 2X2 rotation matrix 
      REAL*8 Th        ! Rotation matrix
!
!  Form the 2X2 rotation matrix (R1) from the sine and cosine
!  | cos(teta)  sin(teta)|
!  |-sin(teta)  cos(teta)|

      R1(1,1)=COS(Th)                                                          
      R1(2,2)=R1(1,1)                                                          
      R1(1,2)=SIN(Th)                                                          
      R1(2,1)=-R1(1,2)
!      write(*,*) R1(1,1),R1(1,2)                                                         
!
!  Normal return
!
      RETURN
      END                                                                   

!------------------------------------------------------------------------
SUBROUTINE Newr(R,R1,N,NN,P,Q)                                         
!
!  Form the new rotation matrix(R)
!  Only update appropriate elements
!  it uses for finding eigen-vector
      IMPLICIT NONE
!
!  Declare variables
!
      INTEGER P,Q       !  Location of maximum off-diagonal element
      INTEGER N         !  Size of input matrix
      INTEGER NN        !  Dimension of input matrix
      INTEGER I         !  Counter
      REAL*8 R(NN,NN)   !  Rotation matrix
      REAL*8 R1(2,2)    !  2X2 rotation matrix
      REAL*8 Rpi        ! Temporarily store a transformed element
!
!  Rotate R to its new form
!
      DO I=1,N                                                             
         Rpi=R1(1,1)*R(P,I)+R1(1,2)*R(Q,I)                                      
         R(Q,I)=R1(2,1)*R(P,I)+R1(2,2)*R(Q,I)                                   
         R(P,I)=Rpi                                                             
      END DO                                                                   
!
!  Normal return
!
      RETURN
      END                                                                   
!---------------------------------------------------------------------------------
SUBROUTINE Newa(A,R1,N,NN,P,Q)
!
!  Form the new input matrix(A(new))
!  A(new)=R1(transpose)*A(old)*R1
!  Only update appropriate elements to save time.
!  if we make R1(transpose)*A(old)*R1 we find following equations for it.
!  To save CPU time for large matrix, we use the following equations in this program
!  otherwise making program for R1(transpose)*A(old)*R1 (however M.JAMAL test it).
!  App(new)=C^2*App(old)+2C*S*Apq(old)+S^2*Aqq(old)   S=sin(theta) , C=cos(theta)
!  Aqq(new)=C^2*Aqq(old)+2C*(-S)*Apq(old)+S^2*App(old)   
!  Apq(new)=Aqp(new)=(C^2-S^2)*Apq(old)+C*S*(Aqq(old)+App(old))   
!  Api(new)=Aip(new)=C*Aip(old)+S*Aiq(old)  i.(notequal).q,p
!  Aqi(new)=Aiq(new)=C*Aiq(old)+(-S)*Aip(old)  i.(notequal).q,p
!  Aij(new)=Aij(old)  i,j.(notequal).q,p 

      IMPLICIT NONE
!
!  Declare variables
!
      INTEGER P,Q       !  Location of maximum off-diagonal element
      INTEGER N         !  Size of input matrix
      INTEGER NN        !  Dimension of input matrix
      INTEGER I         !  Counter
      REAL*8 A(NN,NN)   !  Input matrix
      REAL*8 R1(2,2)    !  2X2 rotation matrix
      REAL*8 Aip,App,Apq,Aqq  ! Temporary variables
!
!  Rotate elements not in 2X2 sub-matrix
!  Api(new)=Aip(new)=C*Aip(old)+S*Aiq(old)  i.(notequal).q,p
!  Aqi(new)=Aiq(new)=C*Aiq(old)+(-S)*Aip(old)  i.(notequal).q,p
      DO I=1,N                                                             
         IF (I.NE.P.AND.I.NE.Q) THEN                                           
            Aip=A(I,P)*R1(1,1)+A(I,Q)*R1(1,2)                                   
            A(I,Q)=A(I,P)*R1(2,1)+A(I,Q)*R1(2,2)                               
            A(I,P)=Aip                                                 
            A(Q,I)=A(I,Q)                                                    
            A(P,I)=A(I,P)                                                     
         END IF
      END DO                      
!
!  Rotate elements in 2X2 sub-matrix
!  App(new)=C^2*App(old)+2C*S*Apq(old)+S^2*Aqq(old)   S=sin(theta) , C=cos(theta)
!  Aqq(new)=C^2*Aqq(old)+2C*(-S)*Apq(old)+S^2*App(old)   
!  Apq(new)=Aqp(new)=(C^2-S^2)*Apq(old)+C*S*(Aqq(old)+App(old))   
      App=A(P,P)*R1(1,1)**2+2.*A(P,Q)*R1(1,1)*R1(1,2)+A(Q,Q)*R1(1,2)**2         
      Aqq=A(P,P)*R1(1,2)**2+2.*A(P,Q)*R1(2,2)*R1(2,1)+A(Q,Q)*R1(2,2)**2         
      Apq=(A(Q,Q)-A(P,P))*R1(1,1)*R1(1,2)+A(P,Q)*(R1(1,1)**2-R1(1,2)**2)        
      A(P,P)=App                                                               
      A(Q,Q)=Aqq                                                               
      A(P,Q)=Apq                                                               
      A(Q,P)=Apq                                                               
!
!  Normal return
!
      RETURN
      END                 
                                                  
!----------------------------------------------------------------------------
SUBROUTINE Jnorm(A,N,NN,E,D)                                  
!
!  Find norm of error
!
      IMPLICIT NONE
!
!  Declare variables
!
      INTEGER I,J         !  Counters
      INTEGER N           !  Size of input matrix
      INTEGER NN          !  Dimension of input matrix
      REAL*8 A(NN,NN)     !  Input matrix
      REAL*8 E            !  Off-diagonal norm
      REAL*8 D            !  Diagonal norm
      D=0.                                           
      E=0.                                             
      DO I=1,N                                           
         DO J=1,N                                          
            IF (I.NE.J) E=E+A(I,J)**2                           
            IF (I.EQ.J) D=D+A(I,J)**2                         
         END DO                                           
      END DO                                           
      RETURN
      END                                                                   


